@@ -121,12 +121,7 @@ class CObject(object):
 
 def is_line_start_of_object(line):
     "Returns True if we are a c object (enum, struct, union)"
-    def find_whole_word(w):
-        return re.compile(r'\b({0})\b'.format(w), flags=re.IGNORECASE).search
-    
-    object_names = ('enum', 'struct', 'union')
-    is_start = [x for x in object_names if find_whole_word(x)(line) != None]
-    return bool(is_start)
+    return bool(re.search('\btypedef struct$|struct$|struct \S*$|enum|union', line))
 
 # This contains all the objects that don't pass convert_to_ctype_object
 NON_CTYPE_OBJ_NAMES = []
@@ -136,7 +131,7 @@ ALL_C_OBJECTS = []
 def get_object_from_name(name):
     global ALL_C_OBJECTS
     for obj in ALL_C_OBJECTS:
-        if name in obj.preferred_name or name in obj.names:
+        if name == obj.preferred_name or name in obj.names:
             return obj
     return None
 
@@ -464,7 +459,7 @@ def parse_header_file(filename):
         last_pos = f.tell()
         c_objects = []
         enum_objects = []
-        pushed_last_pack_size = False
+        pack_size_stack = []
         line = f.readline()
         while line:
             try:
@@ -477,20 +472,21 @@ def parse_header_file(filename):
                 if line.startswith("#"):
                     # get the pack size
                     if re.match('#pragma.*pack.*.*pop.*', line):
-                        if pushed_last_pack_size:
-                            pack_size = last_pack_size
-                            pushed_last_pack_size = False
+                        if pack_size_stack:
+                            pack_size = pack_size_stack.pop()
                         else:
                             pack_size = 0
                     elif re.match('#pragma.*pack.*.*push.*', line) or re.match('#pragma.*pack\(\d\)', line):
+                        pushing = 'push' in line
                         try:
+                            last = pack_size
                             pack_size = int(re.search('\d{1,}', line).group(0))
+                            if pushing:
+                                pack_size_stack.append(last)
                         except AttributeError:
-                            if not 'push' in line:
+                            if not pushing:
                                 raise
-                            pushed_last_pack_size = True
-                            last_pack_size = pack_size
-                            pack_size = 0
+                            pack_size_stack.append(pack_size)
                         if debug_print:
                             print("PACK SIZE:", pack_size)
                     elif re.match('#define.*', line) and len(line.split(' ')) == 3 and not '\\' in line and not 'sizeof' in line:
@@ -545,7 +541,13 @@ def generate(filename='include/ics/icsnVC40.h'):
         shutil.rmtree(output_dir)
     except FileNotFoundError:
         pass
-    ignore_names = ['__fsid_t', '__darwin_pthread_handler_rec', '_mbstate_t' 'NeoDevice', 'neo_device', 'NeoDeviceEx', 'neo_device_ex', 'icsSpyMessage', 'icsSpyMessageJ1850', 'ics_spy_message', 'ics_spy_message_j1850'] 
+    ignore_names = [
+        '__fsid_t', '__darwin_pthread_handler_rec', '_mbstate_t',
+        '_opaque_pthread_attr_t', '_opaque_pthread_cond_t', '_opaque_pthread_condattr_t',
+        '_opaque_pthread_mutex_t', '_opaque_pthread_mutexattr_t', '_opaque_pthread_once_t',
+        '_opaque_pthread_rwlock_t', '_opaque_pthread_rwlockattr_t', '_opaque_pthread_t',
+        'NeoDevice', 'neo_device', 'NeoDeviceEx', 'neo_device_ex',
+        'icsSpyMessage', 'icsSpyMessageJ1850', 'ics_spy_message', 'ics_spy_message_j1850']
     file_names = []
     prefered_names = []
     all_objects = c_objects + enum_objects
