@@ -47,12 +47,12 @@ typedef struct {
 } spy_message_j1850_object;
 
 static PyMemberDef spy_message_object_members[] = {
-    { "StatusBitField", T_ULONG, offsetof(spy_message_object, msg.StatusBitField), 0, "StatusBitField" },
-    { "StatusBitField2", T_ULONG, offsetof(spy_message_object, msg.StatusBitField2), 0, "StatusBitField2" },
-    { "TimeHardware", T_ULONG, offsetof(spy_message_object, msg.TimeHardware), 0, "Hardware time stamp. The TimeStamp is reset on device open" },
-    { "TimeHardware2", T_ULONG, offsetof(spy_message_object, msg.TimeHardware2), 0, "Hardware time stamp. The TimeStamp is reset on device open" },
-    { "TimeSystem", T_ULONG, offsetof(spy_message_object, msg.TimeSystem), 0, "TimeSystem is loaded with the value received from the timeGetTime call in the WIN32 multimedia API." },
-    { "TimeSystem2", T_ULONG, offsetof(spy_message_object, msg.TimeSystem2), 0, "TimeSystem is loaded with the value received from the timeGetTime call in the WIN32 multimedia API." },
+    { "StatusBitField", T_UINT, offsetof(spy_message_object, msg.StatusBitField), 0, "StatusBitField" },
+    { "StatusBitField2", T_UINT, offsetof(spy_message_object, msg.StatusBitField2), 0, "StatusBitField2" },
+    { "TimeHardware", T_UINT, offsetof(spy_message_object, msg.TimeHardware), 0, "Hardware time stamp. The TimeStamp is reset on device open" },
+    { "TimeHardware2", T_UINT, offsetof(spy_message_object, msg.TimeHardware2), 0, "Hardware time stamp. The TimeStamp is reset on device open" },
+    { "TimeSystem", T_UINT, offsetof(spy_message_object, msg.TimeSystem), 0, "TimeSystem is loaded with the value received from the timeGetTime call in the WIN32 multimedia API." },
+    { "TimeSystem2", T_UINT, offsetof(spy_message_object, msg.TimeSystem2), 0, "TimeSystem is loaded with the value received from the timeGetTime call in the WIN32 multimedia API." },
     { "TimeStampHardwareID", T_UBYTE, offsetof(spy_message_object, msg.TimeStampHardwareID), 0, "This is an identifier of what type of hardware timestamp is used. Since neoVI's timestamp is always the same, this doesn't change." },
     { "TimeStampSystemID", T_UBYTE, offsetof(spy_message_object, msg.TimeStampSystemID), 0, "This is an identifier of what type of system timestamp is used. Since WIN32 neoVI's timestamp is always the same, from the timeGetTime API, this doesn't change." },
     { "NetworkID", T_UBYTE, offsetof(spy_message_object, msg.NetworkID), 0, "This value is used to identify which network this message was received on." },
@@ -64,10 +64,10 @@ static PyMemberDef spy_message_object_members[] = {
     { "NumberBytesData", T_UBYTE, offsetof(spy_message_object, msg.NumberBytesData), 0, "Holds the number of bytes in the Data(1 to 8) array or the number of bytes in a CAN remote frame (The DLC)." },
     { "NetworkID2", T_UBYTE, offsetof(spy_message_object, msg.NetworkID2), 0, "This value is used to identify which network this message was received on." },
     { "DescriptionID", T_SHORT, offsetof(spy_message_object, msg.DescriptionID), 0, "Not Used" },
-    { "ArbIDOrHeader", T_LONG, offsetof(spy_message_object, msg.ArbIDOrHeader), 0, "" },
+    { "ArbIDOrHeader", T_UINT, offsetof(spy_message_object, msg.ArbIDOrHeader), 0, "" },
     { "Data", T_OBJECT_EX, NULL, 0, "" },
-    { "StatusBitField3", T_ULONG, offsetof(spy_message_object, msg.StatusBitField3), 0, "StatusBitField3" },
-    { "StatusBitField4", T_ULONG, offsetof(spy_message_object, msg.StatusBitField4), 0, "StatusBitField4" },
+    { "StatusBitField3", T_UINT, offsetof(spy_message_object, msg.StatusBitField3), 0, "StatusBitField3" },
+    { "StatusBitField4", T_UINT, offsetof(spy_message_object, msg.StatusBitField4), 0, "StatusBitField4" },
     { "AckBytes", T_OBJECT_EX, NULL, 0, "" },
     { "ExtraDataPtr", T_OBJECT_EX, offsetof(spy_message_object, msg.ExtraDataPtr), 0, "" },
     { "MiscData", T_UBYTE, offsetof(spy_message_object, msg.MiscData), 0, "" },
@@ -76,12 +76,12 @@ static PyMemberDef spy_message_object_members[] = {
 };
 
 static PyMemberDef spy_message_j1850_object_members[] = {
-    { "StatusBitField", T_ULONG, offsetof(spy_message_j1850_object, msg.StatusBitField), 0, "StatusBitField" },
-    { "StatusBitField2", T_ULONG, offsetof(spy_message_j1850_object, msg.StatusBitField2), 0, "StatusBitField2" },
-    { "TimeHardware", T_ULONG, offsetof(spy_message_j1850_object, msg.TimeHardware), 0, "Hardware time stamp. The TimeStamp is reset on device open" },
-    { "TimeHardware2", T_ULONG, offsetof(spy_message_j1850_object, msg.TimeHardware2), 0, "Hardware time stamp. The TimeStamp is reset on device open" },
-    { "TimeSystem", T_ULONG, offsetof(spy_message_j1850_object, msg.TimeSystem), 0, "TimeSystem is loaded with the value received from the timeGetTime call in the WIN32 multimedia API." },
-    { "TimeSystem2", T_ULONG, offsetof(spy_message_j1850_object, msg.TimeSystem2), 0, "TimeSystem is loaded with the value received from the timeGetTime call in the WIN32 multimedia API." },
+    { "StatusBitField", T_UINT, offsetof(spy_message_j1850_object, msg.StatusBitField), 0, "StatusBitField" },
+    { "StatusBitField2", T_UINT, offsetof(spy_message_j1850_object, msg.StatusBitField2), 0, "StatusBitField2" },
+    { "TimeHardware", T_UINT, offsetof(spy_message_j1850_object, msg.TimeHardware), 0, "Hardware time stamp. The TimeStamp is reset on device open" },
+    { "TimeHardware2", T_UINT, offsetof(spy_message_j1850_object, msg.TimeHardware2), 0, "Hardware time stamp. The TimeStamp is reset on device open" },
+    { "TimeSystem", T_UINT, offsetof(spy_message_j1850_object, msg.TimeSystem), 0, "TimeSystem is loaded with the value received from the timeGetTime call in the WIN32 multimedia API." },
+    { "TimeSystem2", T_UINT, offsetof(spy_message_j1850_object, msg.TimeSystem2), 0, "TimeSystem is loaded with the value received from the timeGetTime call in the WIN32 multimedia API." },
     { "TimeStampHardwareID", T_UBYTE, offsetof(spy_message_j1850_object, msg.TimeStampHardwareID), 0, "This is an identifier of what type of hardware timestamp is used. Since neoVI's timestamp is always the same, this doesn't change." },
     { "TimeStampSystemID", T_UBYTE, offsetof(spy_message_j1850_object, msg.TimeStampSystemID), 0, "This is an identifier of what type of system timestamp is used. Since WIN32 neoVI's timestamp is always the same, from the timeGetTime API, this doesn't change." },
     { "NetworkID", T_UBYTE, offsetof(spy_message_j1850_object, msg.NetworkID), 0, "This value is used to identify which network this message was received on." },
@@ -95,8 +95,8 @@ static PyMemberDef spy_message_j1850_object_members[] = {
     { "DescriptionID", T_SHORT, offsetof(spy_message_j1850_object, msg.DescriptionID), 0, "Not Used" },
     { "Header", T_OBJECT_EX, NULL, 0, "" },
     { "Data", T_OBJECT_EX, NULL, 0, "" },
-    { "StatusBitField3", T_ULONG, offsetof(spy_message_j1850_object, msg.StatusBitField3), 0, "StatusBitField3" },
-    { "StatusBitField4", T_ULONG, offsetof(spy_message_j1850_object, msg.StatusBitField4), 0, "StatusBitField4" },
+    { "StatusBitField3", T_UINT, offsetof(spy_message_j1850_object, msg.StatusBitField3), 0, "StatusBitField3" },
+    { "StatusBitField4", T_UINT, offsetof(spy_message_j1850_object, msg.StatusBitField4), 0, "StatusBitField4" },
     { "AckBytes", T_OBJECT_EX, NULL, 0, "" },
     { "ExtraDataPtr", T_OBJECT_EX, offsetof(spy_message_j1850_object, msg.ExtraDataPtr), 0, "" },
     { "MiscData", T_UBYTE, offsetof(spy_message_j1850_object, msg.MiscData), 0, "" },
@@ -114,8 +114,10 @@ static int spy_message_object_alloc(spy_message_object* self, PyObject* args, Py
 
 static void spy_message_object_dealloc(spy_message_object* self)
 {
-    if (!self->noExtraDataPtrCleanup && self->msg.ExtraDataPtrEnabled && self->msg.ExtraDataPtr != NULL) {
+    if ((!self->noExtraDataPtrCleanup && self->msg.ExtraDataPtrEnabled && self->msg.ExtraDataPtr != NULL) || 
+		(!self->noExtraDataPtrCleanup && self->msg.Protocol == SPY_PROTOCOL_ETHERNET && self->msg.ExtraDataPtr != NULL)) {
         // Clean up the ExtraDataPtr if we can
+        // Ethernet protocol uses the ExtraDataPtrEnabled reversed internally so do a double check here
         delete[] self->msg.ExtraDataPtr;
         self->msg.ExtraDataPtr = NULL;
         self->msg.ExtraDataPtrEnabled = 0;
@@ -184,8 +186,16 @@ static PyObject* spy_message_object_getattr(PyObject *o, PyObject *attr_name)
         spy_message_j1850_object* obj = (spy_message_j1850_object*)o;
         unsigned char *ExtraDataPtr = (unsigned char*)obj->msg.ExtraDataPtr;
         if (obj->msg.ExtraDataPtrEnabled && obj->msg.NumberBytesData && obj->msg.ExtraDataPtr) {
-            PyObject *tuple = PyTuple_New(obj->msg.NumberBytesData);
-            for (int i=0; i < obj->msg.NumberBytesData; ++i) {
+            int actual_size = 0;
+            // Some newer protocols are packing the length into NumberBytesHeader also so lets handle it here...
+            if (obj->msg.Protocol == SPY_PROTOCOL_A2B || obj->msg.Protocol == SPY_PROTOCOL_ETHERNET) {
+                actual_size = (obj->msg.NumberBytesHeader << 8) | obj->msg.NumberBytesData;
+            }
+            else {
+                actual_size = obj->msg.NumberBytesData;
+            }
+            PyObject *tuple = PyTuple_New(actual_size);
+            for (int i=0; i < actual_size; ++i) {
                 PyTuple_SET_ITEM(tuple, i, PyLong_FromLong(ExtraDataPtr[i]));
             }
             return tuple;
@@ -260,8 +270,13 @@ static int spy_message_object_setattr(PyObject *o, PyObject *name, PyObject *val
         }
         return 0;
     }
+    else if (PyUnicode_CompareWithASCIIString(name, "Protocol") == 0) {
+        // Ethernet behavior is backward to CAN and will crash if enabled.
+        if (PyLong_AsLong(value) == SPY_PROTOCOL_ETHERNET)
+            obj->msg.ExtraDataPtrEnabled = 0;
+        return PyObject_GenericSetAttr(o, name, value);
+    }
     else if (PyUnicode_CompareWithASCIIString(name, "ExtraDataPtr") == 0) {
-
         // Make sure we are a tuple and len() == 8
         if (!PyTuple_Check(value)) {
             PyErr_Format(PyExc_AttributeError,
@@ -269,14 +284,14 @@ static int spy_message_object_setattr(PyObject *o, PyObject *name, PyObject *val
                 MODULE_NAME "." SPY_MESSAGE_OBJECT_NAME, name);
             return -1;
         }
-
         // Get tuple items and place them in array, set as 0 if error.
         Py_ssize_t length = PyObject_Length(value);
         if (obj->msg.ExtraDataPtr != NULL)
             delete[] obj->msg.ExtraDataPtr;
         obj->msg.ExtraDataPtr = new unsigned char[PyObject_Length(value)];
         obj->msg.NumberBytesData = PyObject_Length(value);
-        obj->msg.ExtraDataPtrEnabled = 1;
+        if (obj->msg.Protocol != SPY_PROTOCOL_ETHERNET)
+            obj->msg.ExtraDataPtrEnabled = 1;
         unsigned char * ExtraDataPtr = (unsigned char*)(obj->msg.ExtraDataPtr);
         for (int i=0; i < PyObject_Length(value); ++i) {
             PyObject* data = PyTuple_GetItem(value, i);
@@ -290,10 +305,16 @@ static int spy_message_object_setattr(PyObject *o, PyObject *name, PyObject *val
     }
     else if (PyUnicode_CompareWithASCIIString(name, "ExtraDataPtrEnabled") == 0) {
         // Make sure we clean up here so we don't memory leak
-        if (!obj->noExtraDataPtrCleanup && PyLong_AsLong(value) != 1 && obj->msg.ExtraDataPtrEnabled == 1) {
+        if ((!obj->noExtraDataPtrCleanup && PyLong_AsLong(value) != 1 && obj->msg.ExtraDataPtrEnabled == 1) ||
+            (!obj->noExtraDataPtrCleanup && PyLong_AsLong(value) != 1 && obj->msg.Protocol == SPY_PROTOCOL_ETHERNET)) {
             if (obj->msg.ExtraDataPtr != NULL)
                 delete[] obj->msg.ExtraDataPtr;
         }
+        else if (PyLong_AsLong(value) != 0 && obj->msg.Protocol == SPY_PROTOCOL_ETHERNET)
+        {
+            // Ethernet always needs to be set to 0
+            return 0;
+        }
         return PyObject_GenericSetAttr(o, name, value);
     }
     else {
@@ -315,4 +336,4 @@ extern PyTypeObject spy_message_j1850_object_type;
 
 bool setup_spy_message_object(PyObject* module);
 
-#endif // _OBJECT_SPY_MESSAGE_H_
\ No newline at end of file
+#endif // _OBJECT_SPY_MESSAGE_H_
